/* Simple stack illustrating idiomatic use of realloc.  For use with sane,
   a.k.a. "zero-null", memory allocator wrapper in sane.[ch].
   Accompanies "Drill Bits" column in ACM _Queue_ magazine, Jan/Feb 2023.
   Author:  Terence Kelly tpkelly@{ acm.org, cs.princeton.edu, eecs.umich.edu }
*/

#include <stddef.h>  /* for size_t */
#include <stdint.h>  /* for SIZE_MAX */
#include "stack.h"   /* errcodes & prototypes */

#define SANE_STANDARDIZE  /* malloc -> sane_malloc etc. */
#include "sane.h"

#if ! defined __STDC__ || defined  __cplusplus || \
    ! defined SANE_H_INCLUDED
#error "compile as standard C, use sane (zero-null) allocator"
#endif

static int  * S;  /* array of strictly positive ints */
static size_t N;  /* number of ints on stack */

static int resize(const size_t nu) {
  int *t;
  if (nu > SIZE_MAX / sizeof *S)         return TOOBIG;
  t = realloc(S, nu * sizeof *S);
  if (nu && !t)  /* ask >0, get none */  return ALLOCFAIL;
  S = t;
  N = nu;  /* NB: side effect */         return 0;  }

int push(const int val) {                 /* all error */
  int r;                                  /* codes are */
  if (0 >= val)           return BADARG;  /* negative; */
  if ((r = resize(N+1)))  return r;       /* see       */
  S[N-1] = val;           return 0;  }    /* stack.h   */

int pop(void) {
  int r, topval;
  if (!N)                 return EMPTY;
  topval = S[N-1];
  if ((r = resize(N-1)))  return r;
  else                    return topval;  }

